from copy import deepcopy

from mower.data import agent_list
from mower.solvers.fight.battle_choose.battle_filter import BattleFilterSolver
from mower.solvers.fight.battle_choose.battle_tag_choose import BattleTagChoose
from mower.solvers.fight.battle_choose.choose_skill import ChooseSkillSolver
from mower.utils import config
from mower.utils.character_recognize import (
    operator_team_select,
)
from mower.utils.graph.utils import SceneGraphSolver
from mower.utils.image import cropimg
from mower.utils.log import logger
from mower.utils.scene import Scene


class BattleAgentChooseSolver(SceneGraphSolver):
    solver_name = "战斗选择指定干员"

    def run(self, agents: list) -> list:
        if agents == []:
            raise ValueError("干员列表为空")
        self.agents = agents
        self.agents_copy = deepcopy(agents)

        self.agent = None
        self.tmp_swipe = None

        self.choosed = []
        self.lack = []
        self.clean = False
        self.filter = False
        self.tmp_left = None

        super().run()

        self.choosed = set(self.choosed)
        logger.debug(f"选择完成干员：{self.choosed}")
        return [self.lack, len(self.choosed)]

    @property
    def interval(self):
        return config.screenshot_avg / 500  # 截图延迟

    def choose_agents(self):
        if self.agent is None:
            if len(self.agents_copy) > 0:
                """
                self.agent =[干员名,是否选过tag,是否左滑过]
                """
                agent = self.agents_copy.pop(0)
                self.agent = [agent, False, False]
                logger.info(f"开始选择干员：{self.agent[0]}")
            else:
                return True

        agents = operator_team_select(config.recog.img)
        agents_dict = dict(agents)

        if self.tmp_swipe:
            logger.debug(
                f"tmp_swipe:{self.tmp_swipe[-1][1][0][0]}, {agents[-1][1][0][0]}"
            )
            if self.tmp_swipe[-1][0] == agents[-1][0]:
                if self.tmp_swipe[-1][1][0][0] != agents[-1][1][0][0]:
                    self.tmp_swipe = agents
                    return
            self.tmp_swipe = None
        agent = None  # self.agent[0]["name"]

        if "opers" in self.agent[0]:
            for i in self.agent[0]["opers"]:
                if i["name"] in agents_dict.keys():
                    agent = i
                    logger.info(f"干员组选择：{agent}")

            if agent is None:
                agent = self.agent[0]["opers"][0]
        else:
            agent = self.agent[0]

        agent_name = agent["name"]

        if agent_name in agents_dict.keys():
            if self.is_choosed(agents_dict[agent_name]):
                ChooseSkillSolver().run(agent["skill"])
                logger.info(f"{agent}选择完成")
                self.choosed.append(agent_name)
                self.agent = None

            else:
                if agents_dict[agent_name][0][0] > 1650:
                    self.swipe_noinertia((1000, 540), (-500, 0), interval=0.5)
                    return

                logger.debug(f"tap {agent_name} ")
                self.ctap(agents_dict[agent_name], 2, self.interval, agent_name)
            return
        if self.agent[1] is False:
            if agent_name == "阿米娅":
                tag = "ALL"
            else:
                tag = agent_list[agent_name]["profession"]

            BattleTagChoose().run(tag)
            self.agent[1] = True
            return

        if self.agent[2] is False:
            logger.debug(f"左滑 {self.agent}")
            if self.tmp_left is None:
                self.tmp_left = list(agents_dict.keys())[0]
                self.swipe_noinertia((1000, 540), (300, 0), interval=0.5)
                return
            elif self.tmp_left != list(agents_dict.keys())[0]:
                self.swipe_noinertia((1000, 540), (1000, 0), interval=0.5)
                self.tmp_left = list(agents_dict.keys())[0]
                return
            self.agent[2] = True
            return

        # 右滑
        self.before_swipe = agents
        self.swipe_update(
            [(1000, 540), (-2500, 0)],
            [600],
            self.interval,
            interval=self.interval,
        )

        self.tmp_swipe = operator_team_select(config.recog.img)
        logger.debug(f"右滑 {self.before_swipe},{self.tmp_swipe}")
        if self.tmp_swipe:
            if self.tmp_swipe[-1][0] == self.before_swipe[-1][0]:
                if self.tmp_swipe[-1][1][0][0] == self.before_swipe[-1][1][0][0]:
                    logger.error(RuntimeError("滑动失败"))
                    return True
                else:
                    logger.info(f"未选到：{self.agent[0]}")
                    self.tmp_swipe[-1][1][0][0] != self.before_swipe[-1][1][0][0]
                    self.lack.append(self.agent[0])
                    self.agent = None

    def is_choosed(self, scope) -> bool:
        if (
            cropimg(
                config.recog.img,
                [
                    [scope[0][0] + 60, scope[0][1] - 113],
                    [scope[0][0] + 61, scope[0][1] - 112],
                ],
            )[0][0][0]
            < 50
        ) and cropimg(
            config.recog.img,
            [
                [scope[0][0] + 60, scope[1][1] + 216],
                [scope[0][0] + 61, scope[1][1] + 217],
            ],
        )[0][0][0] < 50:
            return True
        return False

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.OPERATOR_AGENT_SELECT:
            if pos := self.find("choose_agent/open_profession"):
                self.ctap(pos, interval=self.interval)
            if self.filter is False:
                if BattleFilterSolver().run("等级", ascending=False):
                    self.filter = True
            if self.clean is False and len(self.agents) > 1:
                agents_dict = dict(operator_team_select(config.recog.img))
                for i in agents_dict:
                    if self.is_choosed(agents_dict[i]):
                        logger.debug(f"开始选人前先清空干员 {i}")
                        self.tap("choose_agent/clear_battle", interval=self.interval)
                        return
                self.clean = True

            try:
                return self.choose_agents()
            except Exception as e:
                logger.exception(e)
                return True
        elif scene == Scene.OPERATOR_SELECT:
            self.tap("choose_agent/fast_select")
        elif scene == Scene.SSS_SQUAD:
            self.tap("sss/choose/fast_select")
        elif scene in self.waiting_scene:
            self.waiting_solver()
        else:
            return True
